@@ -1,9 +1,16 @@
 /* pam_item.c */
 
 /*
- * $Id: pam_item.c,v 1.8 1997/02/15 15:58:49 morgan Exp morgan $
+ * $Id: pam_item.c,v 1.2 1998/12/27 04:34:22 morgan Exp $
  *
  * $Log: pam_item.c,v $
+ * Revision 1.2  1998/12/27 04:34:22  morgan
+ * reverting logging functions within libpam.  Gone are the externally
+ * advertised API replaced by a more simple (libpam internal) one.
+ *
+ * Revision 1.1.1.1  1998/07/12 05:17:15  morgan
+ * Linux PAM sources pre-0.66
+ *
  */
 
 #include <ctype.h>
@@ -100,8 +107,8 @@ int pam_set_item (
     }
     case PAM_CONV:              /* want to change the conversation function */
 	if (item == NULL) {
-	    pam_system_log(pamh, NULL, LOG_ERR,
-			   "pam_set_item: attempt to set conv() to NULL");
+	    _pam_system_log(LOG_ERR,
+			    "pam_set_item: attempt to set conv() to NULL");
 	    retval = PAM_PERM_DENIED;
 	} else {
 	    struct pam_conv *tconv;
@@ -109,8 +116,8 @@ int pam_set_item (
 	    if ((tconv=
 		 (struct pam_conv *) malloc(sizeof(struct pam_conv))
 		) == NULL) {
-		pam_system_log(pamh, NULL, LOG_CRIT,
-			       "pam_set_item: malloc failed for pam_conv");
+		_pam_system_log(LOG_CRIT,
+				"pam_set_item: malloc failed for pam_conv");
 		retval = PAM_BUF_ERR;
 	    } else {
 		memcpy(tconv, item, sizeof(struct pam_conv));
@@ -122,27 +129,6 @@ int pam_set_item (
     case PAM_FAIL_DELAY:
 	pamh->fail_delay.delay_fn_ptr = item;
 	break;
-    case PAM_LOG_STATE:
-    {
-	char *old_ident = pamh->pam_default_log.ident;
-
-	if (item == NULL) {
-	    /* reset the default state */
-	    pamh->pam_default_log.ident = x_strdup(PAM_LOG_STATE_IDENT);
-	    pamh->pam_default_log.option = PAM_LOG_STATE_OPTION;
-	    pamh->pam_default_log.facility = PAM_LOG_STATE_FACILITY;
-	} else {
-	    const struct pam_log_state *state = item;
-
-	    pamh->pam_default_log.ident = x_strdup(state->ident);
-	    pamh->pam_default_log.option = state->option;
-	    pamh->pam_default_log.facility = state->facility;
-	}
-	_pam_overwrite(old_ident);
-	_pam_drop(old_ident);
-
-	break;
-    }
     default:
 	retval = PAM_BAD_ITEM;
     }
@@ -159,8 +145,8 @@ int pam_get_item (
     IF_NO_PAMH("pam_get_item",pamh,PAM_SYSTEM_ERR);
 
     if (item == NULL) {
-	pam_system_log(pamh, NULL, LOG_ERR,
-		       "pam_get_item: nowhere to place requested item");
+	_pam_system_log(LOG_ERR,
+			"pam_get_item: nowhere to place requested item");
 	return PAM_PERM_DENIED;
     }
 
@@ -172,6 +158,7 @@ int pam_get_item (
 	*item = pamh->user;
 	break;
     case PAM_USER_PROMPT:
+	D(("returning user=%s", pamh->user));
 	*item = pamh->prompt;
 	break;
     case PAM_TTY:
@@ -196,9 +183,6 @@ int pam_get_item (
     case PAM_FAIL_DELAY:
 	*item = pamh->fail_delay.delay_fn_ptr;
 	break;
-    case PAM_LOG_STATE:
-	*item = &(pamh->pam_default_log);
-	break;
     default:
 	/* XXX - I made this up */
 	return PAM_BAD_ITEM;
@@ -220,14 +204,12 @@ int pam_get_user(pam_handle_t *pamh, const char **user, const char *prompt)
     IF_NO_PAMH("pam_get_user", pamh, PAM_SYSTEM_ERR);
 
     if (pamh->pam_conversation == NULL) {
-	pam_system_log(pamh, NULL, LOG_ERR,
-		       "pam_get_user: no conv element in pamh");
+	_pam_system_log(LOG_ERR, "pam_get_user: no conv element in pamh");
 	return PAM_SERVICE_ERR;
     }
 
     if (user == NULL) {  /* ensure the the module has suplied a destination */
-	pam_system_log(pamh, NULL, LOG_ERR,
-		       "pam_get_user: nowhere to record username");
+	_pam_system_log(LOG_ERR, "pam_get_user: nowhere to record username");
 	return PAM_PERM_DENIED;
     } else
 	*user = NULL;
@@ -251,7 +233,7 @@ int pam_get_user(pam_handle_t *pamh, const char **user, const char *prompt)
     if (pamh->former.want_user) {
 	/* must have a prompt to resume with */
 	if (! pamh->former.prompt) {
-	    	    pam_system_log(pamh, NULL, LOG_ERR,
+	    	    _pam_system_log(LOG_ERR,
 				   "pam_get_user: failed to resume with prompt"
 			);
 	    return PAM_ABORT;
@@ -259,8 +241,8 @@ int pam_get_user(pam_handle_t *pamh, const char **user, const char *prompt)
 
 	/* must be the same prompt as last time */
 	if (strcmp(pamh->former.prompt, use_prompt)) {
-	    pam_system_log(pamh, NULL, LOG_ERR,
-			   "pam_get_user: resumed with different prompt");
+	    _pam_system_log(LOG_ERR,
+			    "pam_get_user: resumed with different prompt");
 	    return PAM_ABORT;
 	}
 
@@ -309,5 +291,6 @@ int pam_get_user(pam_handle_t *pamh, const char **user, const char *prompt)
 	_pam_drop_reply(resp, 1);
     }
 
+    D(("completed"));
     return retval;        /* pass on any error from conversation */
 }
