@@ -30,7 +30,11 @@
  */
 
 #include <stdio.h>
-#define __USE_BSD
+#ifdef NEED_CRYPT_HEADER
+# include <crypt.h>
+#else
+# define _BSD_SOURCE
+#endif
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -41,10 +45,9 @@
 #include <ctype.h>
 
 extern char *FascistCheck(char *pw, const char *dictpath);
-extern char *crypt(const char *, const char *);
 
 #ifndef CRACKLIB_DICTPATH
-#define CRACKLIB_DICTPATH "/usr/lib/cracklib_dict"
+#define CRACKLIB_DICTPATH "/usr/share/dict/cracklib_dict"
 #endif
 
 #define PROMPT1 "New %s password: "
@@ -83,20 +86,21 @@ static void _pam_log(int err, const char *format, ...)
 /* argument parsing */
 #define PAM_DEBUG_ARG       0x0001
 
-/* module data - AGM: please remove these static variables... PAM was
- * designed to be reentrant based soley on a unique pamh... this
- * breaks that.  */
+/* module data - AGM: */
 
-static int retry_times = 0;
-static int diff_ok = 10;
-static int min_length = 9;
-static int dig_credit = 1;
-static int up_credit = 1;
-static int low_credit = 1;
-static int oth_credit = 1;
-static char prompt_type[BUFSIZ];
+struct cracklib_options {
+	int retry_times;
+	int diff_ok;
+	int min_length;
+	int dig_credit;
+	int up_credit;
+	int low_credit;
+	int oth_credit;
+	int use_authtok;
+	char prompt_type[BUFSIZ];
+};
 
-static int _pam_parse(int argc, const char **argv)
+static int _pam_parse(struct cracklib_options *opt, int argc, const char **argv)
 {
      int ctrl=0;
 
@@ -109,35 +113,37 @@ static int _pam_parse(int argc, const char **argv)
 	 if (!strcmp(*argv,"debug"))
 	     ctrl |= PAM_DEBUG_ARG;
 	 else if (!strncmp(*argv,"type=",5))
-	     strcpy(prompt_type, *argv+5);
+	     strcpy(opt->prompt_type, *argv+5);
 	 else if (!strncmp(*argv,"retry=",6)) {
-	     retry_times = strtol(*argv+6,&ep,10);
-	     if (!ep || (retry_times < 1))
-		 retry_times = 1;
+	     opt->retry_times = strtol(*argv+6,&ep,10);
+	     if (!ep || (opt->retry_times < 1))
+		 opt->retry_times = 1;
 	 } else if (!strncmp(*argv,"difok=",6)) {
-	     diff_ok = strtol(*argv+6,&ep,10);
-	     if (!ep || (diff_ok < 0))
-		 diff_ok = 10;
+	     opt->diff_ok = strtol(*argv+6,&ep,10);
+	     if (!ep || (opt->diff_ok < 0))
+		 opt->diff_ok = 10;
 	 } else if (!strncmp(*argv,"minlen=",7)) {
-	     min_length = strtol(*argv+7,&ep,10);
-	     if (!ep || (min_length < 5))
-		 min_length = 5;
+	     opt->min_length = strtol(*argv+7,&ep,10);
+	     if (!ep || (opt->min_length < 5))
+		 opt->min_length = 5;
 	 } else if (!strncmp(*argv,"dcredit=",8)) {
-	     dig_credit = strtol(*argv+8,&ep,10);
-	     if (!ep || (dig_credit < 0))
-		 dig_credit = 0;
+	     opt->dig_credit = strtol(*argv+8,&ep,10);
+	     if (!ep || (opt->dig_credit < 0))
+		 opt->dig_credit = 0;
 	 } else if (!strncmp(*argv,"ucredit=",8)) {
-	     up_credit = strtol(*argv+8,&ep,10);
-	     if (!ep || (up_credit < 0))
-		 up_credit = 0;
+	     opt->up_credit = strtol(*argv+8,&ep,10);
+	     if (!ep || (opt->up_credit < 0))
+		 opt->up_credit = 0;
 	 } else if (!strncmp(*argv,"lcredit=",8)) {
-	     low_credit = strtol(*argv+8,&ep,10);
-	     if (!ep || (low_credit < 0))
-		 low_credit = 0;
+	     opt->low_credit = strtol(*argv+8,&ep,10);
+	     if (!ep || (opt->low_credit < 0))
+		 opt->low_credit = 0;
 	 } else if (!strncmp(*argv,"ocredit=",8)) {
-	     oth_credit = strtol(*argv+8,&ep,10);
-	     if (!ep || (oth_credit < 0))
-		 oth_credit = 0;
+	     opt->oth_credit = strtol(*argv+8,&ep,10);
+	     if (!ep || (opt->oth_credit < 0))
+		 opt->oth_credit = 0;
+	 } else if (!strncmp(*argv,"use_authtok",11)) {
+		 opt->use_authtok = 1;
 	 } else {
 	     _pam_log(LOG_ERR,"pam_parse: unknown option; %s",*argv);
 	 }
@@ -225,7 +231,7 @@ static int palindrome(const char *old, const char *new)
  * is longer than 20 bytes (MD5)
  */
 
-static int similiar(const char *old, const char *new)
+static int similiar(struct cracklib_options *opt, const char *old, const char *new)
 {
 	int	i, j;
 
@@ -233,7 +239,7 @@ static int similiar(const char *old, const char *new)
 		if (strchr (new, old[i]))
 			j++;
 
- 	if (j >= diff_ok || i >= j * 2)
+ 	if (j >= opt->diff_ok || i >= j * 2)
 		return 0;
 
 	return 1;
@@ -242,7 +248,7 @@ static int similiar(const char *old, const char *new)
 /*
  * a nice mix of characters.
  */
-static int simple(const char *old, const char *new)
+static int simple(struct cracklib_options *opt, const char *old, const char *new)
 {
 	int	digits = 0;
 	int	uppers = 0;
@@ -270,19 +276,19 @@ static int simple(const char *old, const char *new)
 	 * defaults cause the effect to be the same as before the change
 	 */
 
- 	if (digits > dig_credit)
-	    digits = dig_credit;
+ 	if (digits > opt->dig_credit)
+	    digits = opt->dig_credit;
 
- 	if (uppers > up_credit)
-	    uppers = up_credit;
+ 	if (uppers > opt->up_credit)
+	    uppers = opt->up_credit;
 
- 	if (lowers > low_credit)
-	    lowers = low_credit;
+ 	if (lowers > opt->low_credit)
+	    lowers = opt->low_credit;
 
- 	if (others > oth_credit)
-	    others = oth_credit;
+ 	if (others > opt->oth_credit)
+	    others = opt->oth_credit;
 
- 	size = min_length;
+ 	size = opt->min_length;
  	size -= digits;
  	size -= uppers;
  	size -= lowers;
@@ -303,7 +309,7 @@ static char * str_lower(char *string)
 	return string;
 }
 
-static const char * password_check(const char *old, const char *new)
+static const char * password_check(struct cracklib_options *opt, const char *old, const char *new)
 {
 	const char *msg = NULL;
 	char *oldmono, *newmono, *wrapped;
@@ -325,10 +331,10 @@ static const char * password_check(const char *old, const char *new)
 	if (!msg && strcmp(oldmono, newmono) == 0)
 		msg = "case changes only";
 
-	if (!msg && similiar(oldmono, newmono))
+	if (!msg && similiar(opt, oldmono, newmono))
 		msg = "is too similiar to the old one";
 
-	if (!msg && simple(old, new))
+	if (!msg && simple(opt, old, new))
 		msg = "is too simple";
 
 	if (!msg && strstr(wrapped, newmono))
@@ -383,6 +389,7 @@ static const char * check_old_password(const char *forwho, const char *newpass)
 
 static int _pam_unix_approve_pass(pam_handle_t *pamh,
                                   unsigned int ctrl,
+				  struct cracklib_options *opt,
                                   const char *pass_old,
                                   const char *pass_new)
 {
@@ -403,7 +410,7 @@ static int _pam_unix_approve_pass(pam_handle_t *pamh,
      * if one wanted to hardwire authentication token strength
      * checking this would be the place
      */
-    msg = password_check(pass_old,pass_new);
+    msg = password_check(opt, pass_old,pass_new);
     if (!msg) {
 	retval = pam_get_item(pamh, PAM_USER, (const void **)&user);
 	if (retval != PAM_SUCCESS) {
@@ -437,14 +444,23 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
 				int argc, const char **argv)
 {
     unsigned int ctrl;
+    struct cracklib_options options;
+
+    options.retry_times = 1;
+    options.diff_ok = 10;
+    options.min_length = 9;
+    options.dig_credit = 1;
+    options.up_credit = 1;
+    options.low_credit = 1;
+    options.oth_credit = 1;
+    options.use_authtok = 0;
+    memset(options.prompt_type, 0, BUFSIZ);
     
-    retry_times = 1;
-    memset(prompt_type,0,sizeof(prompt_type));
-    ctrl = _pam_parse(argc, argv);
+    ctrl = _pam_parse(&options, argc, argv);
 
     D(("called."));
-    if (!prompt_type[0])
-        strcpy(prompt_type,"UNIX");
+    if (!options.prompt_type[0])
+        strcpy(options.prompt_type,"UNIX");
 
     if (flags & PAM_PRELIM_CHECK) {
         /* Check for passwd dictionary */       
@@ -493,7 +509,7 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
          */
         token1 = token2 = NULL;
         
-        if (!retry_times) {
+        if (!options.retry_times) {
 	    D(("returning %s because maxtries reached",
 	       pam_strerror(pamh, retval)));
             return retval;
@@ -507,33 +523,48 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
          * set PAM_AUTHTOK and return
          */
 
-        /* Prepare to ask the user for the first time */
-        memset(prompt,0,sizeof(prompt));
-        sprintf(prompt,PROMPT1,prompt_type);
-        pmsg[0] = &msg[0];
-        msg[0].msg_style = PAM_PROMPT_ECHO_OFF;
-        msg[0].msg = prompt;
+	if (options.use_authtok == 1) {
+            retval = pam_get_item(pamh, PAM_AUTHTOK, (const void **) &item);
+	    if (retval != PAM_SUCCESS) {
+		/* very strange. */
+		_pam_log(LOG_ALERT
+			,"pam_get_item returned error to pam_cracklib"
+			);
+	    } else if (item != NULL) {      /* we have a password! */
+		token1 = item;
+		item = NULL;
+	    } else {
+		retval = PAM_AUTHTOK_RECOVER_ERR;         /* didn't work */
+	    }
+	} else {
+            /* Prepare to ask the user for the first time */
+            memset(prompt,0,sizeof(prompt));
+            sprintf(prompt,PROMPT1,options.prompt_type);
+            pmsg[0] = &msg[0];
+            msg[0].msg_style = PAM_PROMPT_ECHO_OFF;
+            msg[0].msg = prompt;
 
-        resp = NULL;
-        retval = converse(pamh, ctrl, 1, pmsg, &resp);
-        if (resp != NULL) {
-            /* interpret the response */
-            if (retval == PAM_SUCCESS) {     /* a good conversation */
-                token1 = x_strdup(resp[0].resp);
-                if (token1 == NULL) {
-                    _pam_log(LOG_NOTICE,
-                             "could not recover authentication token 1");
-                    retval = PAM_AUTHTOK_RECOVER_ERR;
+            resp = NULL;
+            retval = converse(pamh, ctrl, 1, pmsg, &resp);
+            if (resp != NULL) {
+                /* interpret the response */
+                if (retval == PAM_SUCCESS) {     /* a good conversation */
+                    token1 = x_strdup(resp[0].resp);
+                    if (token1 == NULL) {
+                        _pam_log(LOG_NOTICE,
+                                 "could not recover authentication token 1");
+                        retval = PAM_AUTHTOK_RECOVER_ERR;
+                    }
                 }
+                /*
+                 * tidy up the conversation (resp_retcode) is ignored
+                 */
+                _pam_drop_reply(resp, 1);
+            } else {
+                retval = (retval == PAM_SUCCESS) ?
+                         PAM_AUTHTOK_RECOVER_ERR:retval ;
             }
-            /*
-             * tidy up the conversation (resp_retcode) is ignored
-             */
-            _pam_drop_reply(resp, 1);
-        } else {
-            retval = (retval == PAM_SUCCESS) ?
-                     PAM_AUTHTOK_RECOVER_ERR:retval ;
-        }
+	}
 
         if (retval != PAM_SUCCESS) {
             if (ctrl && PAM_DEBUG_ARG)
@@ -562,7 +593,7 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
                 /* check it for strength too... */
 		D(("for strength"));
                 if (oldtoken) {
-                    retval = _pam_unix_approve_pass(pamh,ctrl,
+                    retval = _pam_unix_approve_pass(pamh,ctrl,&options,
                                                oldtoken,token1);
                     if (retval != PAM_SUCCESS) {
                         if (getuid() || (flags & PAM_CHANGE_EXPIRED_AUTHTOK))
@@ -585,77 +616,79 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t *pamh, int flags,
         }
 
         /* Now we have a good passwd. Ask for it once again */
-        
-        bzero(prompt,sizeof(prompt));
-        sprintf(prompt,PROMPT2,prompt_type);
-        pmsg[0] = &msg[0];
-        msg[0].msg_style = PAM_PROMPT_ECHO_OFF;
-        msg[0].msg = prompt;
 
-        resp = NULL;
-        retval = converse(pamh, ctrl, 1, pmsg, &resp);
-        if (resp != NULL) {
-            /* interpret the response */
-            if (retval == PAM_SUCCESS) {     /* a good conversation */
-                token2 = x_strdup(resp[0].resp);
-                if (token2 == NULL) {
-                    _pam_log(LOG_NOTICE,
-                             "could not recover authentication token 2");
-                    retval = PAM_AUTHTOK_RECOVER_ERR;
+        if (options.use_authtok == 0) {
+            bzero(prompt,sizeof(prompt));
+            sprintf(prompt,PROMPT2,options.prompt_type);
+            pmsg[0] = &msg[0];
+            msg[0].msg_style = PAM_PROMPT_ECHO_OFF;
+            msg[0].msg = prompt;
+
+            resp = NULL;
+            retval = converse(pamh, ctrl, 1, pmsg, &resp);
+            if (resp != NULL) {
+                /* interpret the response */
+                if (retval == PAM_SUCCESS) {     /* a good conversation */
+                    token2 = x_strdup(resp[0].resp);
+                    if (token2 == NULL) {
+                        _pam_log(LOG_NOTICE,
+                                 "could not recover authentication token 2");
+                        retval = PAM_AUTHTOK_RECOVER_ERR;
+                    }
                 }
+                /*
+                 * tidy up the conversation (resp_retcode) is ignored
+                 */
+	        _pam_drop_reply(resp, 1);
+            } else {
+                retval = (retval == PAM_SUCCESS) ?
+                         PAM_AUTHTOK_RECOVER_ERR:retval ;
             }
-            /*
-             * tidy up the conversation (resp_retcode) is ignored
+
+            if (retval != PAM_SUCCESS) {
+                if (ctrl && PAM_DEBUG_ARG)
+                    _pam_log(LOG_DEBUG
+			     ,"unable to obtain the password a second time");
+                continue;
+            }
+
+            /* Hopefully now token1 and token2 the same password ... */
+            if (strcmp(token1,token2) != 0) {
+                /* tell the user */
+                make_remark(pamh, ctrl, PAM_ERROR_MSG, MISTYPED_PASS);
+                token1 = _pam_delete(token1);
+                token2 = _pam_delete(token2);
+                pam_set_item(pamh, PAM_AUTHTOK, NULL);
+                if (ctrl & PAM_DEBUG_ARG)
+                    _pam_log(LOG_NOTICE,"Password mistyped");
+                retval = PAM_AUTHTOK_RECOVER_ERR;
+                continue;
+            }
+        
+            /* Yes, the password was typed correct twice
+             * we store this password as an item
              */
-	    _pam_drop_reply(resp, 1);
-        } else {
-            retval = (retval == PAM_SUCCESS) ?
-                     PAM_AUTHTOK_RECOVER_ERR:retval ;
-        }
 
-        if (retval != PAM_SUCCESS) {
-            if (ctrl && PAM_DEBUG_ARG)
-                _pam_log(LOG_DEBUG
-			 ,"unable to obtain the password a second time");
-            continue;
-        }
-
-        /* Hopefully now token1 and token2 the same password ... */
-        if (strcmp(token1,token2) != 0) {
-            /* tell the user */
-            make_remark(pamh, ctrl, PAM_ERROR_MSG, MISTYPED_PASS);
+            retval = pam_set_item(pamh, PAM_AUTHTOK, token1);
+            /* clean it up */
             token1 = _pam_delete(token1);
             token2 = _pam_delete(token2);
-            pam_set_item(pamh, PAM_AUTHTOK, NULL);
-            if (ctrl & PAM_DEBUG_ARG)
-                _pam_log(LOG_NOTICE,"Password mistyped");
-            retval = PAM_AUTHTOK_RECOVER_ERR;
-            continue;
+            if (
+                (retval != PAM_SUCCESS) ||
+                (
+                 (
+                    retval = pam_get_item(pamh, PAM_AUTHTOK, (const void **)&item)
+                 ) != PAM_SUCCESS
+                )
+               ) {
+                    _pam_log(LOG_CRIT, "error manipulating password");
+                    continue;
+            }
+            item = NULL;                 /* break link to password */
+            return PAM_SUCCESS;
         }
         
-        /* Yes, the password was typed correct twice
-         * we store this password as an item
-         */
-
-        retval = pam_set_item(pamh, PAM_AUTHTOK, token1);
-        /* clean it up */
-        token1 = _pam_delete(token1);
-        token2 = _pam_delete(token2);
-        if (
-            (retval != PAM_SUCCESS) ||
-            (
-             (
-                retval = pam_get_item(pamh, PAM_AUTHTOK, (const void **)&item)
-             ) != PAM_SUCCESS
-            )
-           ) {
-                _pam_log(LOG_CRIT, "error manipulating password");
-                continue;
-        }
-        item = NULL;                 /* break link to password */
-        return PAM_SUCCESS;
-        
-        } while (retry_times--);
+        } while (options.retry_times--);
 
     } else {
         if (ctrl & PAM_DEBUG_ARG)
