@@ -1,13 +1,16 @@
 /* pam_mail module */
 
 /*
- * $Id: pam_env.c,v 1.1.1.1 1998/07/12 05:17:16 morgan Exp $
+ * $Id: pam_env.c,v 1.2 1999/10/09 05:08:28 morgan Exp $
  *
  * Written by Dave Kinchlea <kinch@kinch.ark.com> 1997/01/31
  * Inspired by Andrew Morgan <morgan@parc.power.net, who also supplied the 
  * template for this file (via pam_mail)
  *
  * $Log: pam_env.c,v $
+ * Revision 1.2  1999/10/09 05:08:28  morgan
+ * removed libpwdb dependencies (libpwdb support is depreciated)
+ *
  * Revision 1.1.1.1  1998/07/12 05:17:16  morgan
  * Linux PAM sources pre-0.66
  *
@@ -70,10 +73,6 @@
 #include <sys/types.h>
 #include <unistd.h>
 
-#ifdef WANT_PWDB
-#include <pwdb/pwdb_public.h>
-#endif
-
 /*
  * here, we make a definition for the externally accessible function
  * in this file (this definition is required for static a module
