@@ -1,5 +1,5 @@
 /*
- * $Id: upperLOWER.c,v 1.1.1.1 1998/07/12 05:17:16 morgan Exp $
+ * $Id: upperLOWER.c,v 1.2 1999/07/08 05:01:48 morgan Exp $
  *
  * This is a sample filter program, for use with pam_filter (a module
  * provided with Linux-PAM). This filter simply transposes upper and
@@ -7,6 +7,9 @@
  * it serves no purpose other than to annoy the user...
  *
  * $Log: upperLOWER.c,v $
+ * Revision 1.2  1999/07/08 05:01:48  morgan
+ * glibc fixes (Thorsten Kukuk, Adam J. Richter)
+ *
  * Revision 1.1.1.1  1998/07/12 05:17:16  morgan
  * Linux PAM sources pre-0.66
  *
@@ -15,6 +18,11 @@
  *
  */
 
+#ifdef linux
+# define _GNU_SOURCE
+# include <features.h>
+#endif
+
 #include <stdio.h>
 #include <syslog.h>
 #include <sys/time.h>
