@@ -1,20 +1,9 @@
 /* pam_ftp module */
 
 /*
- * $Id: pam_ftp.c,v 1.1.1.1 1998/07/12 05:17:17 morgan Exp $
- *
- * Written by Andrew Morgan <morgan@parc.power.net> 1996/3/11
- *
- * $Log: pam_ftp.c,v $
- * Revision 1.1.1.1  1998/07/12 05:17:17  morgan
- * Linux PAM sources pre-0.66
- *
- * Revision 1.2  1997/02/15 16:23:59  morgan
- * fixed logging to avoid a fixed buffer size
- *
- * Revision 1.1  1996/12/01 03:17:57  morgan
- * Initial revision
+ * $Id: pam_ftp.c,v 1.2 1999/10/09 05:10:21 morgan Exp $
  *
+ * Written by Andrew Morgan <morgan@linux.kernel.org> 1996/3/11
  *
  */
 
@@ -217,7 +206,7 @@ int pam_sm_authenticate(pam_handle_t *pamh,int flags,int argc
 	mesg[i] = &msg[i];
 	msg[i].msg_style = PAM_PROMPT_ECHO_OFF;
 	if (anon) {
-	    prompt = malloc(sizeof(PLEASE_ENTER_PASSWORD + strlen(user)));
+	    prompt = malloc(sizeof(PLEASE_ENTER_PASSWORD) + strlen(user));
 	    sprintf(prompt, PLEASE_ENTER_PASSWORD, user);
 	    msg[i].msg = prompt;
 	} else {
@@ -225,8 +214,10 @@ int pam_sm_authenticate(pam_handle_t *pamh,int flags,int argc
 	}
 
 	retval = converse(pamh, ++i, mesg, &resp);
-	_pam_overwrite(prompt);
-	_pam_drop(prompt);
+	if (prompt) {
+	    _pam_overwrite(prompt);
+	    _pam_drop(prompt);
+	}
 
 	if (retval != PAM_SUCCESS) {
 	    if (resp != NULL)
@@ -241,11 +232,15 @@ int pam_sm_authenticate(pam_handle_t *pamh,int flags,int argc
 		token = strtok(resp->resp, "@");
 		retval = pam_set_item(pamh, PAM_RUSER, token);
 
-		if (token && retval != PAM_SUCCESS) {
+		if ((token) && (retval == PAM_SUCCESS)) {
 		    token = strtok(NULL, "@");
 		    retval = pam_set_item(pamh, PAM_RHOST, token);
 		}
 	    }
+
+	    /* we are happy to grant annonymous access to the user */
+	    retval = PAM_SUCCESS;
+
 	} else {
 	    /*
 	     * we have a password so set AUTHTOK
