@@ -17,8 +17,10 @@
 #error THIS CODE IS KNOWN TO WORK ONLY ON LINUX !!!
 #endif 
 
-# define _GNU_SOURCE
-# include <features.h>
+#define _GNU_SOURCE
+#define _BSD_SOURCE
+
+#include <features.h>
 
 #include <stdio.h>
 #include <unistd.h>
@@ -38,6 +40,9 @@
 #define UT_USER ut_user
 #endif
 
+#include <grp.h>
+#include <pwd.h>
+
 /* Module defines */
 #define LINE_LENGTH 1024
 
@@ -74,7 +79,6 @@ static int priority;	    /* the priority to run user process with */
 
 #include <security/pam_modules.h>
 #include <security/_pam_macros.h>
-#include <pwdb/pwdb_map.h>
 
 /* logging */
 static void _pam_log(int err, const char *format, ...)
