@@ -1,5 +1,5 @@
 /*
- * $Id: pwdb_chkpwd.c,v 1.1.1.1 1998/07/12 05:17:17 morgan Exp $
+ * $Id: pwdb_chkpwd.c,v 1.2 1999/07/08 05:02:03 morgan Exp $
  *
  * This program is designed to run setuid(root) or with sufficient
  * privilege to read all of the unix password databases. It is designed
@@ -12,6 +12,9 @@
  * Copyright information is located at the end of the file.
  *
  * $Log: pwdb_chkpwd.c,v $
+ * Revision 1.2  1999/07/08 05:02:03  morgan
+ * glibc fixes (Thorsten Kukuk, Adam J. Richter)
+ *
  * Revision 1.1.1.1  1998/07/12 05:17:17  morgan
  * Linux PAM sources pre-0.66
  *
@@ -95,7 +98,7 @@ static int _unix_verify_passwd(const char *salt, const char *p)
     return retval;
 }
 
-void main(void)
+int main(void)
 {
     const struct pwdb *pw=NULL;
     const struct pwdb_entry *pwe=NULL;
