@@ -6,7 +6,10 @@
 #include <features.h>
 
 #include <stdio.h>
+
+#ifndef __USE_POSIX2
 #define __USE_POSIX2
+#endif /* __USE_POSIX2 */
 
 #include <stdlib.h>
 
