@@ -103,6 +103,7 @@ int innetgr(const char *, const char *, const char *,const char *);
 
 struct _options {
     int  opt_no_hosts_equiv;
+    int  opt_hosts_equiv_rootok;
     int  opt_no_rhosts;
     int  opt_debug;
     int  opt_nowarn;
@@ -111,6 +112,8 @@ struct _options {
     int  opt_promiscuous;
     int  opt_suppress;
     int  opt_private_group;
+    int  opt_no_uid_check;
+    const char *superuser;
     const char *last_error;
 };
 
@@ -133,6 +136,11 @@ static void set_option (struct _options *opts, const char *arg)
 	return;
     }
 
+    if (strcmp(arg, "hosts_equiv_rootok") == 0) {
+        opts->opt_hosts_equiv_rootok = 1;
+        return;
+    }
+
     if (strcmp(arg, "no_rhosts") == 0) {
 	opts->opt_no_rhosts = 1;
 	return;
@@ -165,6 +173,15 @@ static void set_option (struct _options *opts, const char *arg)
 	return;
     }
 
+    if (strcmp(arg, "no_uid_check") == 0) {
+	opts->opt_no_uid_check = 1;  /* NIS optimization */
+	return;
+    }
+
+    if (strcmp(arg, "superuser=") == 0) {
+	opts->superuser = arg+sizeof("superuser=")-1;
+	return;
+    }
     /*
      * All other options are ignored at the present time.
      */
@@ -444,7 +461,7 @@ pam_iruserok(pam_handle_t *pamh,
     int answer;
     char pbuf[MAXPATHLEN];               /* potential buffer overrun */
 
-    if ( !superuser && !opts->opt_no_hosts_equiv ) {
+    if ((!superuser||opts->opt_hosts_equiv_rootok) && !opts->opt_no_hosts_equiv ) {
 
 	/* try to open system hosts.equiv file */
 	hostf = fopen (_PATH_HEQUIV, "r");
@@ -677,8 +694,12 @@ static int _pam_auth_rhosts (pam_handle_t *pamh,
 	    break;
 	}
 
+	if (opts.superuser && !strcmp(opts.superuser, luser)) {
+	    as_root = 1;
+	}
+
 	/* check if the luser uid == 0... --cristiang */
-	{
+	if (! opts.opt_no_uid_check) {
 	    struct passwd *luser_pwd;
 
 	    luser_pwd = getpwnam(luser);
@@ -762,6 +783,16 @@ struct pam_module _pam_rhosts_auth_modstruct = {
 
 /*
  * $Log: pam_rhosts_auth.c,v $
+ * Revision 1.3  1999/10/09 05:12:49  morgan
+ * added hosts_equiv_rootok support
+ *
+ * Revision 1.2  1998/12/14 05:47:58  morgan
+ * added a couple of options: specify the name of root, and don't do the
+ * user-known check.
+ *
+ * Revision 1.1.1.1  1998/07/12 05:17:16  morgan
+ * Linux PAM sources pre-0.66
+ *
  * Revision 1.12  1997/09/27 14:34:01  morgan
  * fixed comment and renamed iruserok to pam_iruserok.
  *
