@@ -1,4 +1,4 @@
-/* pam_securetty module */
+/* pam_shells module */
 
 #define SHELL_FILE "/etc/shells"
 
@@ -8,6 +8,8 @@
  * This code shamelessly ripped from the pam_securetty module.
  */
 
+#define _BSD_SOURCE
+
 #include <pwd.h>
 #include <stdarg.h>
 #include <stdio.h>
