@@ -1,7 +1,7 @@
 /*
  * pam_tally.c
  * 
- * $Id: pam_tally.c,v 1.5 1999/07/04 23:53:38 morgan Exp $
+ * $Id: pam_tally.c,v 1.6 1999/10/09 05:15:17 morgan Exp $
  */
 
 
@@ -17,7 +17,7 @@
 #endif
 
 #include <stdio.h>
-#include <strings.h>
+#include <string.h>
 #include <unistd.h>
 #include <stdarg.h>
 #include <syslog.h>
