@@ -3,5 +3,5 @@ list=`/bin/ls /lib/libnsl.so.* 2> /dev/null`
 if [ -z "$list" ]; then
    echo no
 else
-   exit yes
+   echo yes
 fi
