@@ -84,7 +84,7 @@ PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t * pamh, int flags,
 
 	D(("called."));
 
-	ctrl = _set_ctrl(flags, argc, argv);
+	ctrl = _set_ctrl(flags, NULL, argc, argv);
 
 	retval = pam_get_item(pamh, PAM_USER, (const void **) &uname);
 	D(("user = `%s'", uname));
