@@ -104,7 +104,7 @@ PAM_EXTERN int pam_sm_authenticate(pam_handle_t * pamh, int flags
 
 	D(("called."));
 
-	ctrl = _set_ctrl(flags, argc, argv);
+	ctrl = _set_ctrl(flags, NULL, argc, argv);
 
 	/* get the user'name' */
 
@@ -193,7 +193,7 @@ PAM_EXTERN int pam_sm_setcred(pam_handle_t * pamh, int flags
 	   only argument we need is UNIX_LIKE_AUTH: if it was set,
 	   pam_get_data will succeed. If it wasn't, it will fail, and we
 	   return PAM_SUCCESS.  -SRL */
-	ctrl = _set_ctrl(flags, argc, argv);
+	ctrl = _set_ctrl(flags, NULL, argc, argv);
 	retval = PAM_SUCCESS;
 
 	if (on(UNIX_LIKE_AUTH, ctrl)) {
