@@ -99,10 +99,6 @@ extern int getrpcport(const char *host, unsigned long prognum,
 
 extern char *bigcrypt(const char *key, const char *salt);
 
-/* FIXME: we should not be using global variables: not thread-safe. */
-
-extern int remember;
-
 /*
    How it works:
    Gets in username (has to be done) from the calling program
@@ -435,7 +431,7 @@ static int _update_shadow(const char *forwho, char *towhat)
 }
 
 static int _do_setpass(const char *forwho, char *fromwhat, char *towhat,
-		       unsigned int ctrl)
+		       unsigned int ctrl, int remember)
 {
 	struct passwd *pwd = NULL;
 	int retval = 0;
@@ -649,6 +645,7 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 {
 	unsigned int ctrl, lctrl;
 	int retval;
+	int remember = -1;
 
 	/* <DO NOT free() THESE> */
 	const char *user;
@@ -663,7 +660,7 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 	   and deadlock. */
 	lckpwdf();
 #endif
-	ctrl = _set_ctrl(flags, argc, argv);
+	ctrl = _set_ctrl(flags, &remember, argc, argv);
 
 	/*
 	 * First get the name of a user
@@ -954,7 +951,7 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 
 		/* update the password database(s) -- race conditions..? */
 
-		retval = _do_setpass(user, pass_old, tpass, ctrl);
+		retval = _do_setpass(user, pass_old, tpass, ctrl, remember);
 		_pam_overwrite(pass_new);
 		_pam_overwrite(pass_old);
 		_pam_delete(tpass);
